#include "bw.h"

//------------------------------------------------------------------------
//
// BW.CPP - Source of BetterView library.
//
// VERSION NUMBER: 2.1 (with styles, effects and window scrolling)
//
// BetterView - Library to make interface of your win32 program better.
//
//------------------------------------------------------------------------

// NOTES: 
// If radio buttons are combined with a groupbox control,
// that groupbox MUST have a higher tab order,
// in order that controls work correctly.

// build options:
//   BW_OPTION_NO_HINTS  - do not support hints.
//   BW_OPTION_NO_COLORCHOOSERS - do not support color-choosing controls.

#define BW_PROP_TYPE			"BW_PROP_TYPE"
#define BW_PROP_COLOR			"BW_PROP_COLOR"
#define BW_PROP_BACKCOLOR		"BW_PROP_BACKCOLOR"
#define BW_PROP_BACK			"BW_PROP_BACK"
#define BW_PROP_WNDPROC			"BW_PROP_WNDPROC"
#define BW_PROP_STYLE			"BW_PROP_STYLE"
#define BW_PROP_HINT			"BW_PROP_HINT"
#define BW_PROP_HINTDEST		"BW_PROP_HINTDEST"
#define BW_PROP_DEFAULTHINT		"BW_PROP_DEFAULTHINT"

#define BW_WINDOW_BACK			0x696969 

#define BW_GROUP_BORDER			0x507FFF
#define BW_GROUP_TEXT			0xFFFFFF

#define BW_BUTTON_BACK			0x507FFF
#define BW_BUTTON_TEXT			0xFFFFFF
#define BW_BUTTON_BORDER		0x000000
#define BW_BUTTON_DISABLED_BACK	0xA9A9A9	//0x696969

#define BW_BUTTON_LIGHTBACK		0xED9564	//0xAACD66	//0xFACE87	//0x90EE90	//0x8BBC8F	//0xED9564

#define BW_STATIC_COLOR			0xFFFFFF
#define BW_CHECKBOX_COLOR		0x507FFF
#define BW_CHECKBOX_LIGHTCOLOR	0xED9564
#define BW_EDIT_TEXT_COLOR		0xFF0000

#ifndef IDC_HAND
	#define IDC_HAND            MAKEINTRESOURCE(32649)
#endif

#define BW_EFFECT_TIMER			11023

HWND bwLightButton = NULL;

bwStyleStruct bwCurrentStyle;

// SOME PROTOTYPES:

LRESULT CALLBACK bwOnMsg(HWND, UINT, WPARAM, LPARAM); // Process messages for a window.

//------------------------------------------------------------------------
// CALLBACK FUNCTION FOR LineDDA FOR BUTTONS:
//------------------------------------------------------------------------
void CALLBACK bwLineDDAProc(int x, int y, LPARAM data)
{
	struct bwDDA
	{
		HDC hdc;
		COLORREF color;
	} *p = (bwDDA *) data;

	if ((x + y) & 2) SetPixel(p->hdc, x, y, p->color);
}

//------------------------------------------------------------------------
// FUNCTION TO DRAW DOTTED RECTANGLE:
//------------------------------------------------------------------------
void DottedRectangle(HDC hdc, RECT &rc, COLORREF color)
{
	struct
	{
		HDC hdc;
		COLORREF color;
	} data = { hdc, color };

	LineDDA(rc.left, rc.top, rc.right, rc.top, bwLineDDAProc, (LPARAM) &data);
	LineDDA(rc.right, rc.top, rc.right, rc.bottom, bwLineDDAProc, (LPARAM) &data);
	LineDDA(rc.right, rc.bottom, rc.left, rc.bottom, bwLineDDAProc, (LPARAM) &data);
	LineDDA(rc.left, rc.bottom, rc.left, rc.top, bwLineDDAProc, (LPARAM) &data);
}

//------------------------------------------------------------------------
// DRAW BUTTONS:
//------------------------------------------------------------------------
void bwDrawButton(LPDRAWITEMSTRUCT lpdis)
{
	HWND hParentWnd = GetParent(lpdis->hwndItem);
	RECT rc;
	CopyRect(&rc, &lpdis->rcItem);
	bwStyleStruct *pSt = (bwStyleStruct*) GetProp(hParentWnd, BW_PROP_STYLE);

	HDC hdc = lpdis->hDC;

	COLORREF backcolor = (lpdis->itemState & ODS_DISABLED)?pSt->ButtonDisabledBack:((lpdis->hwndItem == bwLightButton)?pSt->ButtonLightBack:pSt->ButtonBack);
	HBRUSH hBack = CreateSolidBrush(backcolor);
	HPEN hBorder = CreatePen(PS_SOLID, 1, pSt->ButtonBorder);

	HGDIOBJ hBackOld   = SelectObject(hdc, hBack),
			hBorderOld = SelectObject(hdc, hBorder);

	Rectangle(hdc, rc.left, rc.top, rc.right, rc.bottom);

	if (lpdis->itemState & ODS_FOCUS)
	{
		InflateRect(&rc, -2, -2);
		DottedRectangle(hdc, rc, pSt->ButtonText);
		InflateRect(&rc, 2, 2);
	}

	SelectObject(hdc, hBackOld);
	SelectObject(hdc, hBorderOld);

	DeleteObject(hBack);
	DeleteObject(hBorder);

	COLORREF TextColorOld = SetTextColor(hdc, pSt->ButtonText),
			 BackColorOld = SetBkColor(hdc, backcolor);

	char text [128];

	if (lpdis->itemState & ODS_SELECTED) OffsetRect(&rc, 1, 1);

	DrawText(hdc, text, GetWindowText(lpdis->hwndItem, text, 127), &rc, DT_CENTER | DT_VCENTER | DT_SINGLELINE);

	SetTextColor(hdc, TextColorOld);
	SetBkColor(hdc, BackColorOld);
}

LRESULT CALLBACK bwButtonWndProc(HWND This, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	switch (Msg)
	{
	case WM_SETCURSOR:
		SetCursor(LoadCursor(NULL, IDC_HAND));
		break;

	case WM_MOUSEMOVE:
		if (bwLightButton != This)
		{
			if (bwLightButton)
			{
				HWND hRdw = bwLightButton;
				bwLightButton = NULL;
				RedrawWindow(hRdw, NULL, NULL, RDW_INVALIDATE|RDW_UPDATENOW);
			}
			bwLightButton = This;
			RedrawWindow(This, NULL, NULL, RDW_INVALIDATE|RDW_UPDATENOW);

#ifndef BW_OPTION_NO_HINTS
			bwChangeCurrentHint(GetParent(This), This);
#endif

		}

	default:
		return CallWindowProc((WNDPROC) GetProp(This, BW_PROP_WNDPROC), This, Msg, wParam, lParam);
	}
	return FALSE;
}

//------------------------------------------------------------------------
// DRAW GROUPS:
//------------------------------------------------------------------------
void bwDrawGroup(LPDRAWITEMSTRUCT lpdis)
{
	HWND hParentWnd = GetParent(lpdis->hwndItem);
	bwStyleStruct *pSt = (bwStyleStruct*) GetProp(hParentWnd, BW_PROP_STYLE);
	RECT rc;
	CopyRect(&rc, &lpdis->rcItem);
	InflateRect(&rc, -1, -1);

	HDC hdc = lpdis->hDC;

	char text [128];
	SIZE size;

	GetTextExtentPoint32(hdc, text, GetWindowText(lpdis->hwndItem, text, 127), &size);

	rc.top += size.cy / 2;

	DottedRectangle(hdc, rc, pSt->GroupBorder);

	COLORREF TextColorOld = SetTextColor(hdc, pSt->GroupText),
			 BackColorOld = SetBkColor(hdc, pSt->WindowBack);

	TextOut(hdc, rc.left + size.cy, rc.top - size.cy/2, text, lstrlen(text));

	SetTextColor(hdc, TextColorOld);
	SetBkColor(hdc, BackColorOld);
}

LRESULT CALLBACK bwGroupWndProc(HWND This, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	switch (Msg)
	{
	case WM_ERASEBKGND:
		return TRUE;

	case WM_MOUSEMOVE:
		if (bwLightButton)
		{
			HWND hRdw = bwLightButton;
			bwLightButton = NULL;
			RedrawWindow(hRdw, NULL, NULL, RDW_INVALIDATE|RDW_UPDATENOW);

#ifndef BW_OPTION_NO_HINTS
			bwChangeCurrentHint(GetParent(This), NULL);
#endif

		}

	default:
		return CallWindowProc((WNDPROC) GetProp(This, BW_PROP_WNDPROC), This, Msg, wParam, lParam);
	}
//	return FALSE;
}

//------------------------------------------------------------------------
// DRAW WINDOW:
//------------------------------------------------------------------------
void OnPaint(HWND hWnd)
{
	bwStyleStruct *pSt = (bwStyleStruct*) GetProp(hWnd, BW_PROP_STYLE);
	PAINTSTRUCT ps;
	RECT rc;

	GetClientRect(hWnd, &rc);
	HBRUSH hBack = CreateSolidBrush(pSt->WindowBack);

	HDC hdc = BeginPaint(hWnd, &ps);
	HGDIOBJ hBackOld = SelectObject(hdc, hBack);

	FillRect(hdc, &rc, hBack);

	SelectObject(hdc, hBackOld);
	EndPaint(hWnd, &ps);
	DeleteObject(hBack);
}

//------------------------------------------------------------------------
// WNDPROC for choose_color control (static with text "$$COLOR").
//------------------------------------------------------------------------
LRESULT CALLBACK bwColorWndProc(HWND This, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	switch (Msg)
	{
	case WM_SETCURSOR:
		SetCursor(LoadCursor(NULL, IDC_HAND));
		break;

	case WM_MOUSEMOVE:
		if (bwLightButton)
		{
			HWND hRdw = bwLightButton;
			bwLightButton = NULL;
			RedrawWindow(hRdw, NULL, NULL, RDW_INVALIDATE|RDW_UPDATENOW);
		}

	default:
		return CallWindowProc((WNDPROC) GetProp(This, BW_PROP_WNDPROC), This, Msg, wParam, lParam);
	}
	return FALSE;
}

//------------------------------------------------------------------------
// INSTALL BW-LIB ON A WINDOW:
//------------------------------------------------------------------------
void bwInstallLoop(HWND hWnd, bwStyleStruct *pSt)
{
	// Check class name of window:
	char class_name [MAX_PATH+1];

	// Get class name:
	if (GetClassName(hWnd, class_name, MAX_PATH))
	{
		WNDPROC NewWndProc = NULL;
		//LONG NewStyle = 0;

		SetProp(hWnd, BW_PROP_TYPE, 0);

		if (!lstrcmpi(class_name, "BUTTON"))
		{
			LONG OldStyle = GetWindowLong(hWnd, GWL_STYLE);

			switch (OldStyle & 0x0F)
			{
			case BS_PUSHBUTTON:
			case BS_DEFPUSHBUTTON:
				NewWndProc = bwButtonWndProc;
				SetWindowLong(hWnd, GWL_STYLE, BS_OWNERDRAW | OldStyle);
				SetProp(hWnd, BW_PROP_TYPE, (HANDLE) BS_PUSHBUTTON);
				break;

			case BS_GROUPBOX:
				NewWndProc = bwGroupWndProc;
				SetWindowLong(hWnd, GWL_STYLE, BS_OWNERDRAW | (BS_GROUPBOX ^ OldStyle));
				SetProp(hWnd, BW_PROP_TYPE, (HANDLE) BS_GROUPBOX);
				break;

			case BS_AUTOCHECKBOX:
			case BS_AUTORADIOBUTTON:
				NewWndProc = bwButtonWndProc;
				SetProp(hWnd, BW_PROP_COLOR, (HANDLE) pSt->CheckboxText);
				SetProp(hWnd, BW_PROP_BACKCOLOR, (HANDLE) pSt->WindowBack);
				SetProp(hWnd, BW_PROP_BACK, (HANDLE) CreateSolidBrush(pSt->WindowBack));
				break;
			}
		}
		else if (!lstrcmpi(class_name, "STATIC"))
		{
#ifndef BW_OPTION_NO_COLORCHOOSERS
			char txt [16];
			GetWindowText(hWnd, txt, 15);
			if (!lstrcmp(txt, "$$COLOR"))
			{
				SetWindowText(hWnd, "");
				NewWndProc = bwColorWndProc;
				SetProp(hWnd, BW_PROP_COLOR, 0);
				SetProp(hWnd, BW_PROP_BACKCOLOR, 0);
				SetProp(hWnd, BW_PROP_BACK, (HANDLE) CreateSolidBrush(0));
				SetWindowLong(hWnd, GWL_STYLE, SS_NOTIFY | GetWindowLong(hWnd, GWL_STYLE));
				SetProp(hWnd, BW_PROP_TYPE, (HANDLE) -1);
			}
			else
			{
#endif
				SetProp(hWnd, BW_PROP_COLOR, (HANDLE) pSt->StaticText);
				SetProp(hWnd, BW_PROP_BACKCOLOR, (HANDLE) pSt->WindowBack);
				SetProp(hWnd, BW_PROP_BACK, (HANDLE) CreateSolidBrush(pSt->WindowBack));
#ifndef BW_OPTION_NO_COLORCHOOSERS
			}
#endif
		}

		if (NewWndProc)
			SetProp(hWnd, BW_PROP_WNDPROC,
				(HANDLE) SetWindowLong(hWnd, GWL_WNDPROC, (LONG) NewWndProc));
	}

	// Install bw-lib on child windows:
	for (HWND hChild = GetWindow(hWnd, GW_CHILD); hChild; hChild = GetWindow(hChild, GW_HWNDNEXT))
		bwInstallLoop(hChild, pSt);
}

void bwInit()
{
	// Set style struct:
	bwCurrentStyle.ButtonBack			= BW_BUTTON_BACK;
	bwCurrentStyle.ButtonBorder			= BW_BUTTON_BORDER;
	bwCurrentStyle.ButtonDisabledBack	= BW_BUTTON_DISABLED_BACK;
	bwCurrentStyle.ButtonLightBack		= BW_BUTTON_LIGHTBACK;
	bwCurrentStyle.ButtonText			= BW_BUTTON_TEXT;
	bwCurrentStyle.CheckboxText			= BW_CHECKBOX_COLOR;
	bwCurrentStyle.CheckboxLightText	= BW_CHECKBOX_LIGHTCOLOR;
	bwCurrentStyle.GroupBorder			= BW_GROUP_BORDER;
	bwCurrentStyle.GroupText			= BW_GROUP_TEXT;
	bwCurrentStyle.StaticText			= BW_STATIC_COLOR;
	bwCurrentStyle.WindowBack			= BW_WINDOW_BACK;
	bwCurrentStyle.EditText				= BW_EDIT_TEXT_COLOR;
}

void bwInstall(HWND hWnd)
{
	bwInstall(hWnd, &bwCurrentStyle);
}

void bwInstall(HWND hWnd, bwStyleStruct *pSt)
{
	// Subclass wndproc:
	register HANDLE WndProc = (HANDLE) GetWindowLong(hWnd, GWL_WNDPROC);
	if (!WndProc) MessageBox(NULL, "Could not subclass a window!", "bw: error", MB_ICONERROR|MB_OK|MB_TOPMOST);
	SetProp(hWnd, BW_PROP_WNDPROC, WndProc);
	SetProp(hWnd, BW_PROP_STYLE, (HANDLE) pSt);
	SetWindowLong(hWnd, GWL_WNDPROC, (LONG) bwOnMsg);
	bwInstallLoop(hWnd, pSt);
}

void bwInstallWithEffect(HWND hWnd, void (*Effect)(HWND))
{
	bwInstallWithEffect(hWnd, &bwCurrentStyle, Effect);
}

void bwInstallWithEffect(HWND hWnd, bwStyleStruct *pSt, void (*Effect)(HWND))
{
	bwInstall(hWnd, pSt);
	Effect(hWnd);
}

//------------------------------------------------------------------------
// FREE USED MEMORY FOR A WINDOW:
//------------------------------------------------------------------------
void bwFreeMemory(HWND hWnd)
{
	HANDLE hBrush = GetProp(hWnd, BW_PROP_BACK);
	if (hBrush) DeleteObject((HBRUSH) hBrush);

	RemoveProp(hWnd, BW_PROP_TYPE);
	RemoveProp(hWnd, BW_PROP_COLOR);
	RemoveProp(hWnd, BW_PROP_BACKCOLOR);
	RemoveProp(hWnd, BW_PROP_BACK);
//	RemoveProp(hWnd, BW_PROP_WNDPROC);

#ifndef BW_OPTION_NO_HINTS
	RemoveProp(hWnd, BW_PROP_DEFAULTHINT);
	RemoveProp(hWnd, BW_PROP_HINT);
	RemoveProp(hWnd, BW_PROP_HINTDEST);
#endif

	// Free memory for all the children:
	for (HWND hChild = GetWindow(hWnd, GW_CHILD); hChild; hChild = GetWindow(hChild, GW_HWNDNEXT))
		bwFreeMemory(hChild);
}

//------------------------------------------------------------------------
// Change general colors of a control.
//------------------------------------------------------------------------
void bwChangeColorsOfControl(HWND hWnd, COLORREF Text, COLORREF Back)
{
	SetProp(hWnd, BW_PROP_COLOR, (HANDLE) Text);
	SetProp(hWnd, BW_PROP_BACKCOLOR, (HANDLE) Back);
	DeleteObject((HGDIOBJ) GetProp(hWnd, BW_PROP_BACK));
	SetProp(hWnd, BW_PROP_BACK, (HANDLE) CreateSolidBrush(Back));
}

//------------------------------------------------------------------------
// PROCESS DIALOG MESSAGES:
//------------------------------------------------------------------------
LRESULT CALLBACK bwOnMsg(HWND This, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	switch (Msg)
	{
	case WM_PAINT:
		OnPaint(This);
		return FALSE;

	case WM_DRAWITEM:
		switch (((LPDRAWITEMSTRUCT) lParam)->CtlType)
		{
		case ODT_BUTTON:
			switch ((LONG) GetProp(((LPDRAWITEMSTRUCT) lParam)->hwndItem, BW_PROP_TYPE))
			{
			case BS_PUSHBUTTON:
				bwDrawButton((LPDRAWITEMSTRUCT) lParam);
				return TRUE;

			case BS_GROUPBOX:
				bwDrawGroup((LPDRAWITEMSTRUCT) lParam);
				return TRUE;
			}
			break;
		}
		return FALSE;

	case WM_CTLCOLORSTATIC:
	case WM_CTLCOLORBTN:
//	case WM_CTLCOLORLISTBOX:
//	case WM_CTLCOLORSCROLLBAR:
		{
			bwStyleStruct *pSt = (bwStyleStruct*) GetProp(This, BW_PROP_STYLE);
			char clss[8];

			GetClassName((HWND)lParam, clss, 7);

			if (!lstrcmpi(clss, "EDIT"))
			{
				SetTextColor((HDC) wParam, (COLORREF) pSt->EditText);
				//SetBkMode((HDC) wParam, TRANSPARENT);
				SetBkColor((HDC) wParam, GetSysColor(COLOR_3DFACE));
			}
			else
			{
				if ((HWND)lParam == bwLightButton && !GetProp((HWND)lParam, BW_PROP_TYPE))
					SetTextColor((HDC) wParam, (COLORREF) pSt->CheckboxLightText);
				else
					SetTextColor((HDC) wParam, (COLORREF) GetProp((HWND) lParam, BW_PROP_COLOR));

				SetBkColor((HDC) wParam, (COLORREF) pSt->WindowBack);
			}
		}
		//SetBkColor((HDC) wParam, (COLORREF) GetProp((HWND) lParam, BW_PROP_BACKCOLOR));
//		SetBkMode((HDC) wParam, TRANSPARENT);
		return (LRESULT) GetProp((HWND) lParam, BW_PROP_BACK);

	case WM_CTLCOLOREDIT:
		{
			bwStyleStruct *pSt = (bwStyleStruct*) GetProp(This, BW_PROP_STYLE);

			SetBkColor((HDC) wParam, 0xFFFFFF);
			SetTextColor((HDC) wParam, pSt->EditText);
		}
		return (LRESULT) GetStockObject(WHITE_BRUSH);

	case WM_DESTROY:
		bwFreeMemory(This);
		break;

	case WM_MOUSEMOVE:
		if (bwLightButton)
		{
			HWND hRdw = bwLightButton;
			bwLightButton = NULL;
			RedrawWindow(hRdw, NULL, NULL, RDW_INVALIDATE|RDW_UPDATENOW);

#ifndef BW_OPTION_NO_HINTS
			bwChangeCurrentHint(This, NULL);
#endif

		}
		break;

#ifndef BW_OPTION_NO_COLORCHOOSERS
	case WM_COMMAND:
		if (GetProp((HWND)lParam, BW_PROP_TYPE) == (HANDLE) -1)
		{
			static COLORREF custclrs [16] = 
			{
				0x000000, 0x800000, 0x008000, 0x808000,
				0x000080, 0x800080, 0x008080, 0x808080,
				0x404040, 0xFF0000, 0x00FF00, 0xFFFF00,
				0x0000FF, 0xFF00FF, 0x00FFFF, 0xFFFFFF,
			};

			CHOOSECOLOR cc;

			cc.lStructSize	= sizeof CHOOSECOLOR;
			cc.Flags		= CC_RGBINIT;
			cc.hwndOwner	= This;
			cc.lpCustColors	= custclrs;
			cc.rgbResult    = (COLORREF) GetProp((HWND) lParam, BW_PROP_BACKCOLOR);

			if (ChooseColor(&cc))
			{
				bwChangeColorsOfControl((HWND) lParam, cc.rgbResult, cc.rgbResult);
				RedrawWindow((HWND)lParam, NULL, NULL, RDW_INVALIDATE|RDW_UPDATENOW);
			}
		}
		break;
#endif

	}
	register WNDPROC WndProc = (WNDPROC) GetProp(This, BW_PROP_WNDPROC);
	if (WndProc) return CallWindowProc(WndProc, This, Msg, wParam, lParam);
	return DefWindowProc(This, Msg, wParam, lParam);
}

//------------------------------------------------------------------------
// BW WINDOW-SHOW EFFECTS:
//------------------------------------------------------------------------
int bwEffectParam;

void CALLBACK bwEffect_RollOut_UpDown_Proc(HWND hWnd, UINT Msg, UINT idEvent, DWORD dwTime)
{
	RECT rc;
	GetWindowRect(hWnd, &rc);
	rc.right -= rc.left;
	rc.bottom -= rc.top;
	HRGN hRgn = CreateRectRgn(0, 0, rc.right, bwEffectParam);
	SetWindowRgn(hWnd, hRgn, TRUE);
	if (bwEffectParam > rc.bottom)
	{
		KillTimer(hWnd, BW_EFFECT_TIMER);
		SetWindowRgn(hWnd, NULL, TRUE);
	}
	else bwEffectParam += 8;
}

void bwEffect_RollOut_UpDown(HWND hWnd)
{
	bwEffectParam = 0;
	SetWindowRgn(hWnd, CreateRectRgn(0, 0, 0, 0), TRUE);
	SetTimer(hWnd, BW_EFFECT_TIMER, 1, bwEffect_RollOut_UpDown_Proc);
}

void CALLBACK bwEffect_RollOut_LeftRight_Proc(HWND hWnd, UINT Msg, UINT idEvent, DWORD dwTime)
{
	RECT rc;
	GetWindowRect(hWnd, &rc);
	rc.right -= rc.left;
	rc.bottom -= rc.top;
	HRGN hRgn = CreateRectRgn(0, 0, bwEffectParam, rc.bottom);
	SetWindowRgn(hWnd, hRgn, TRUE);
	if (bwEffectParam > rc.right)
	{
		KillTimer(hWnd, BW_EFFECT_TIMER);
		SetWindowRgn(hWnd, NULL, TRUE);
	}
	else bwEffectParam += 8;
}

void bwEffect_RollOut_LeftRight(HWND hWnd)
{
	bwEffectParam = 0;
	SetWindowRgn(hWnd, CreateRectRgn(0, 0, 0, 0), TRUE);
	SetTimer(hWnd, BW_EFFECT_TIMER, 1, bwEffect_RollOut_LeftRight_Proc);
}

void CALLBACK bwEffect_RollOut_Center_Proc(HWND hWnd, UINT Msg, UINT idEvent, DWORD dwTime)
{
	RECT rc;
	GetWindowRect(hWnd, &rc);
	rc.right -= rc.left;
	rc.bottom -= rc.top;
	rc.right /= 2;
	rc.bottom /= 2;
	int mx = (rc.right > rc.bottom)?rc.right:rc.bottom;
	HRGN hRgn = CreateRectRgn(rc.right-bwEffectParam, rc.bottom-bwEffectParam,
					rc.right+bwEffectParam, rc.bottom+bwEffectParam);
	SetWindowRgn(hWnd, hRgn, TRUE);
	if (bwEffectParam > mx)
	{
		KillTimer(hWnd, BW_EFFECT_TIMER);
		SetWindowRgn(hWnd, NULL, TRUE);
	}
	else bwEffectParam += 8;
}

void bwEffect_RollOut_Center(HWND hWnd)
{
	bwEffectParam = 0;
	SetWindowRgn(hWnd, CreateRectRgn(0, 0, 0, 0), TRUE);
	SetTimer(hWnd, BW_EFFECT_TIMER, 1, bwEffect_RollOut_Center_Proc);
}

//------------------------------------------------------------------------
// Set new window style.
//------------------------------------------------------------------------
void bwSetWindowStyle(HWND hWnd, bwStyleStruct *NewStyle)
{
	SetProp(hWnd, BW_PROP_STYLE, (HANDLE) NewStyle);
	RedrawWindow(hWnd, NULL, NULL, RDW_INVALIDATE|RDW_UPDATENOW);
}

//------------------------------------------------------------------------
// Get color of $$COLOR control.
//------------------------------------------------------------------------
COLORREF bwGetDlgItemColor(HWND hDlg, int id)
{
	return (COLORREF) GetProp(GetDlgItem(hDlg, id), BW_PROP_BACKCOLOR);
}

//------------------------------------------------------------------------
// Function to scroll window content.
//
// Example of use:
//
//	case WM_SIZE:
//	case WM_VSCROLL:
//	case WM_HSCROLL:
//		bwScrollWindowSupport(This, Msg, wParam, lParam, &WinSize);
//		break;
//
//	case WM_INITDIALOG:
//		GetClientRect(This, &WinSize);
//		bwCheckMaxWindowSize(This);
//
//------------------------------------------------------------------------
void bwScrollWindowSupport(HWND This, UINT Msg, WPARAM wParam, LPARAM lParam, LPRECT WinSize)
{
	switch (Msg)
	{
	case WM_SIZE:
		if (wParam != SIZE_MINIMIZED)
		{
			ScrollWindow(This, GetScrollPos(This, SB_HORZ),
				GetScrollPos(This, SB_VERT), NULL, NULL);

			SetScrollPos(This, SB_VERT, 0, TRUE);
			SetScrollPos(This, SB_HORZ, 0, TRUE);

			if (HIWORD(lParam) < WinSize->bottom - 4)
			{
				SetWindowLong(This, GWL_STYLE, WS_VSCROLL | GetWindowLong(This, GWL_STYLE));
				SetScrollRange(This, SB_VERT, 0, WinSize->bottom - HIWORD(lParam), TRUE);
			}
			else
			{
				SetWindowLong(This, GWL_STYLE, ~WS_VSCROLL & GetWindowLong(This, GWL_STYLE));
				SetScrollRange(This, SB_VERT, 0, 0, TRUE);
			}

			if (LOWORD(lParam) < WinSize->right - 4)
			{
				SetWindowLong(This, GWL_STYLE, WS_HSCROLL | GetWindowLong(This, GWL_STYLE));
				SetScrollRange(This, SB_HORZ, 0, WinSize->right - LOWORD(lParam), TRUE);
			}
			else
			{
				SetWindowLong(This, GWL_STYLE, ~WS_HSCROLL & GetWindowLong(This, GWL_STYLE));
				SetScrollRange(This, SB_HORZ, 0, 0, TRUE);
			}
		}
		break;

	case WM_VSCROLL:
		{
			int vpos = GetScrollPos(This, SB_VERT);
			int tmp, vmax;

			GetScrollRange(This, SB_VERT, &tmp, &vmax);

			switch (LOWORD(wParam))
			{
			case SB_LINEUP:
				if (vpos > 0)
				{
					ScrollWindow(This, 0, 1, NULL, NULL);
					SetScrollPos(This, SB_VERT, vpos-1, TRUE);
				}
				break;

			case SB_LINEDOWN:
				if (vpos < vmax)
				{
					ScrollWindow(This, 0, -1, NULL, NULL);
					SetScrollPos(This, SB_VERT, vpos+1, TRUE);
				}
				break;

			case SB_PAGEUP:
				if (vpos > 0)
				{
					if (vpos < 8) tmp = vpos; else tmp = 8;
					ScrollWindow(This, 0, tmp, NULL, NULL);
					SetScrollPos(This, SB_VERT, vpos-tmp, TRUE);
				}
				break;

			case SB_PAGEDOWN:
				if (vpos < vmax)
				{
					if (vmax-vpos < 8) tmp = vmax-vpos; else tmp = 8;
					ScrollWindow(This, 0, -tmp, NULL, NULL);
					SetScrollPos(This, SB_VERT, vpos+tmp, TRUE);
				}
				break;

			case SB_THUMBTRACK:
				ScrollWindow(This, 0, vpos-HIWORD(wParam), NULL, NULL);
				SetScrollPos(This, SB_VERT, HIWORD(wParam), TRUE);
				break;
			}
		}
		break;

	case WM_HSCROLL:
		{
			int hpos = GetScrollPos(This, SB_HORZ);
			int tmp, hmax;

			GetScrollRange(This, SB_HORZ, &tmp, &hmax);

			switch (LOWORD(wParam))
			{
			case SB_LINEUP:
				if (hpos > 0)
				{
					ScrollWindow(This, 1, 0, NULL, NULL);
					SetScrollPos(This, SB_HORZ, hpos-1, TRUE);
				}
				break;

			case SB_LINEDOWN:
				if (hpos < hmax)
				{
					ScrollWindow(This, -1, 0, NULL, NULL);
					SetScrollPos(This, SB_HORZ, hpos+1, TRUE);
				}
				break;

			case SB_PAGEUP:
				if (hpos > 0)
				{
					if (hpos < 8) tmp = hpos; else tmp = 8;
					ScrollWindow(This, tmp, 0, NULL, NULL);
					SetScrollPos(This, SB_HORZ, hpos-tmp, TRUE);
				}
				break;

			case SB_PAGEDOWN:
				if (hpos < hmax)
				{
					if (hmax-hpos < 8) tmp = hmax-hpos; else tmp = 8;
					ScrollWindow(This, -tmp, 0, NULL, NULL);
					SetScrollPos(This, SB_HORZ, hpos+tmp, TRUE);
				}
				break;

			case SB_THUMBTRACK:
				ScrollWindow(This, hpos-HIWORD(wParam), 0, NULL, NULL);
				SetScrollPos(This, SB_HORZ, HIWORD(wParam), TRUE);
				break;
			}
		}
		break;
	}
}

void bwCheckMaxWindowSize(HWND This)
{
	RECT rc, rcdesk;
	GetWindowRect(This, &rc);
	SystemParametersInfo(SPI_GETWORKAREA, 0, &rcdesk, 0);
	int maxhei = (rcdesk.bottom-rcdesk.top);
	if (rc.bottom-rc.top > maxhei)
	{
		SetWindowPos(This, HWND_TOP,
			rc.left,
			(rcdesk.bottom-rcdesk.top-maxhei)/2,
			rc.right-rc.left + GetSystemMetrics(SM_CYHSCROLL),
			maxhei, 0);
	}
}

//------------------------------------------------------------------------
// Set hint.
//------------------------------------------------------------------------
#ifndef BW_OPTION_NO_HINTS

void bwSetHint(HWND This, int idCtrl, const char *HintStr)
{
	SetProp(GetDlgItem(This, idCtrl), BW_PROP_HINT, (HANDLE) HintStr);
}

void bwSetHintDisplayControl(HWND This, int idCtrl, const char *DefaultHint)
{
	HWND Dest = GetDlgItem(This, idCtrl);
	SetProp(This, BW_PROP_HINTDEST, (HANDLE) Dest);
	SetProp(Dest, BW_PROP_DEFAULTHINT, (HANDLE) DefaultHint);
}

void bwChangeCurrentHint(HWND hParent, HWND hControl)
{
	HWND Dest = (HWND) GetProp(hParent, BW_PROP_HINTDEST);
	if (hControl)
	{
		HANDLE Hint = GetProp(hControl, BW_PROP_HINT);
		SetWindowText(Dest, (char *) (Hint?Hint:GetProp(Dest, BW_PROP_DEFAULTHINT)));
	}
	else SetWindowText(Dest, (char *) GetProp(Dest, BW_PROP_DEFAULTHINT));
}

#endif
//------------------------------------------------------------------------
// END OF BW.CPP
//------------------------------------------------------------------------
