// included into installer.cpp

// Controlling functions for ReplaceFile dialog.

static BOOL ReplaceAlwaysYes = FALSE;
static BOOL ReplaceAlwaysNo  = FALSE;

typedef struct replacefiledata__
{
	char *filename;
	DWORD newsize;
}
*replacefiledata_p, replacefiledata_t;

BOOL CALLBACK ReplaceFileDlgProc(HWND This, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	switch (Msg)
	{
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
			EndDialog(This, TRUE);
			return TRUE;

		case IDCANCEL:
			EndDialog(This, FALSE);
			return TRUE;

		case IDC_ALWAYS_YES:
			ReplaceAlwaysYes = TRUE;
			EndDialog(This, TRUE);
			return TRUE;

		case IDC_ALWAYS_NO:
			ReplaceAlwaysNo = TRUE;
			EndDialog(This, FALSE);
			return TRUE;
		}
		break;

	case WM_INITDIALOG:
		UseLang(mls_replace, sizeof(mls_replace)/sizeof(mls_replace[0]), This);
		{
			char cnvsize [64];
			replacefiledata_p prfd = (replacefiledata_p) lParam;
			SetDlgItemText(This, IDC_FILENAME, prfd->filename);
			convert_size_to_string(cnvsize, prfd->newsize);
			SetDlgItemText(This, IDC_NEWSIZE, cnvsize);
		}
#ifndef DISABLE_BW
		if (!(Flags & FLAG_STYLE_WINDOWS)) bwInstall(This, &UserStyle);
#endif
		return TRUE;
	}
	return FALSE;
}

int RunReplaceFileDialog(HWND Parent, char *filename, DWORD newsize)
{
	if (ReplaceAlwaysYes) return TRUE;
	if (ReplaceAlwaysNo)  return FALSE;
	replacefiledata_t rfd;
	rfd.filename = filename;
	rfd.newsize  = newsize;
	return DialogBoxParam(hInstance, (LPCTSTR) IDD_REPLACE_FILE, Parent, ReplaceFileDlgProc, (LPARAM) &rfd);
}

// LINKS DIALOG:
static int allflags[6] =
{ FLAG_CHECK_DESKTOP, FLAG_CHECK_STARTMENU,   FLAG_CHECK_PROGRAMS,
  FLAG_CHECK_STARTUP, FLAG_CHECK_QUICKLAUNCH, FLAG_CHECK_SENDTO
};

static int alllinks[6] =
{ IDC_LINK_DESKTOP, IDC_LINK_STARTMENU,  IDC_LINK_PROGRAMS,
  IDC_LINK_STARTUP, IDC_LINK_QUICKLAUNCH, IDC_LINK_SENDTO
};


BOOL CALLBACK LinksDlg(HWND This, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	switch (Msg)
	{
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
		{
			for (int i = 0; i < 6; ++i)
				if (BST_CHECKED == IsDlgButtonChecked(This, alllinks[i]))
					CurrentFlags |= allflags[i];
				else 
					CurrentFlags &= ~allflags[i];
		}

		case IDCANCEL:
			EndDialog(This, 0);
			return TRUE;
		}
		break;

	case WM_INITDIALOG:
		UseLang(mls_links, sizeof(mls_links)/sizeof(mls_links[0]), This);
		{
			for (int i = 0; i < 6; ++i)
			{
				EnableWindow(GetDlgItem(This, alllinks[i]), (Flags & allflags[i]));
				CheckDlgButton(This, alllinks[i],
					(CurrentFlags & allflags[i])?BST_CHECKED:BST_UNCHECKED);
			}
		}
#ifndef DISABLE_BW
		if (!(Flags & FLAG_STYLE_WINDOWS)) bwInstall(This, &UserStyle);
#endif
		return TRUE;
	}
	return FALSE;
}

BOOL CALLBACK ChooseILang(HWND This, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	switch (Msg)
	{
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_ILANG_RU:
		case IDC_ILANG_EN:
			EndDialog(This, LOWORD(wParam));
			return TRUE;
		}
		break;

	case WM_INITDIALOG:
#ifndef DISABLE_BW
		if (!(Flags & FLAG_STYLE_WINDOWS)) bwInstall(This, &UserStyle);
#endif
		return TRUE;
	}
	return FALSE;
}




BOOL CALLBACK PasswdDlg(HWND This, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	static char *valid_passwd = "";

	switch (Msg)
	{
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
			{
				char *ttt = get_text(GetDlgItem(This, IDC_PASSWD));
				int passed = !lstrcmp(valid_passwd, ttt);
				LocalFree(ttt);
				EndDialog(This, passed);
			}
			return TRUE;

		case IDCANCEL:
			EndDialog(This, 0);
			return TRUE;
		}
		break;

	case WM_INITDIALOG:
		UseLang(mls_passwd, sizeof(mls_passwd)/sizeof(mls_passwd[0]), This);
#ifndef DISABLE_BW
		if (!(Flags & FLAG_STYLE_WINDOWS)) bwInstall(This, &UserStyle);
#endif
		valid_passwd = (char *)lParam;
		return TRUE;
	}
	return FALSE;
}


