/*
** tiny C library for Windows.
** It replaces some standard C functions with Win32 APIs and adds new ones.
** (C) 2007, Alexander G. Karpov a.k.a. mrhx.
*/

#include <windows.h>

#define END_OF_STREAM (-1)

typedef struct STREAM_
{
	unsigned long pos, size;
	unsigned char *pmem;
	int flags;
} STREAM;

#define STREAM_READ			1
#define STREAM_WRITE		2
#define STREAM_OVERFLOW		4
#define STREAM_END			8

int stream_putc(int ch, STREAM *ps)
{
	if (!(ps->flags & STREAM_WRITE))
	{
		MessageBox(NULL, "stream_putc: not writable", "tinyclib", MB_OK|MB_ICONERROR|MB_TOPMOST);
		ExitProcess(1001);
	}
	else if (ps->pos >= ps->size)
	{
//		char info [512];
//		wsprintf(info, "stream_putc: overflow\n" "tag: %s\n" "size: %i", ps->tag?ps->tag:"NULL", ps->size);
//		MessageBox(NULL, info, "tinyclib", MB_OK|MB_ICONERROR|MB_TOPMOST);
//		ExitProcess(1002);
		ps->flags |= STREAM_OVERFLOW;
	}
	else
	{
		ps->pmem [ps->pos] = ch;
		++ps->pos;
	}
	return ch;
}

void printf(const char *msg, ...)
{
	char buf [1024];
	wvsprintf(buf, msg, (va_list)(&msg+1));
	MessageBox(NULL, buf, NULL, MB_OK|MB_ICONERROR|MB_TOPMOST);
}

int stream_getc(STREAM *ps)
{
	int ch;
	if (!(ps->flags & STREAM_READ))
	{
		MessageBox(NULL, "stream_getc: not readable", "tinyclib", MB_OK|MB_ICONERROR|MB_TOPMOST);
		ExitProcess(1003);
	}
	if (ps->pos >= ps->size) ch = END_OF_STREAM;
	else
	{
		ch = ps->pmem [ps->pos];
		++ps->pos;
	}
	//printf("getc  %i  %X  %c", ch, ch, ch);
	return ch;
}

STREAM * stream_init_mem(STREAM *ps, unsigned char *pmem, unsigned long size, int flags)
{
	ps->pmem = pmem;
	ps->pos = 0;
	ps->size = size;
	ps->flags = flags;
	return ps;
}

void exit(int code)
{
	ExitProcess(code);
}

void free(void *ptr)
{
	LocalFree((HLOCAL) ptr);
}

void * malloc(unsigned int size)
{
	void *ptr = LocalAlloc(LMEM_FIXED, size);
	if (!ptr)
	{
		MessageBox(NULL, "malloc: no memory", "tinyclib", MB_OK|MB_ICONERROR|MB_TOPMOST);
		ExitProcess(0);
	}
	return ptr;
}
